;*************************************************************** 
;* Feladat: rjon egy programot az AVR Experiment krtyra, 
;  amely kb. msodperces temezssel villogtat egy LED -et. 
;  Az idoztst egy SW idozto rutinnal vgezze. Egy nyomgomb 
;  megnyomsa utn a LED villogs helyett folyamatosan gjen. 
;  Ugyanazon nyomgombot mg egyszer megnyomva megint villogjon. 
;  A nyomgomb pergsmentestst oldja meg. 
;
;* Rvid lers: LED villogtats-2
; 
;* Szerzk: Okos Dniel
;* Mrcsoport: <merocsoport jele>
;
;***************************************************************
;* "AVR ExperimentBoard" port assignment information:
;***************************************************************
;*
;* LED0(P):PortC.0          LED4(P):PortC.4
;* LED1(P):PortC.1          LED5(P):PortC.5
;* LED2(S):PortC.2          LED6(S):PortC.6
;* LED3(Z):PortC.3          LED7(Z):PortC.7        INT:PortE.4
;*
;* SW0:PortG.0     SW1:PortG.1     SW2:PortG.4     SW3:PortG.3
;* 
;* BT0:PortE.5     BT1:PortE.6     BT2:PortE.7     BT3:PortB.7
;*
;***************************************************************
;*
;* AIN:PortF.0     NTK:PortF.1    OPTO:PortF.2     POT:PortF.3
;*
;***************************************************************
;*
;* LCD1(VSS) = GND         LCD9(DB2): -
;* LCD2(VDD) = VCC         LCD10(DB3): -
;* LCD3(VO ) = GND         LCD11(DB4): PortA.4
;* LCD4(RS ) = PortA.0     LCD12(DB5): PortA.5
;* LCD5(R/W) = GND         LCD13(DB6): PortA.6
;* LCD6(E  ) = PortA.1     LCD14(DB7): PortA.7
;* LCD7(DB0) = -           LCD15(BLA): VCC
;* LCD8(DB1) = -           LCD16(BLK): PortB.5 (1=Backlight ON)
;*
;***************************************************************

.include "m128def.inc" ; Definition file for ATmega128 
;* Program Constants 
.equ const =$00 ; Generic Constant Structure example  
;* Program Variables Definitions 
.def temp 	=r16 ; Temporary Register example 
.def cntr1	=r17 ; Szmll a 4ms-os temezshez
.def cntr2	=r18 ; Szmll a msodperces temezshez
.def flash_on 	=r19 ; Villogs engedlyezo/tilt "flag"
.def pressed	=r20 ; Gomb lenyomst jelzo "flag"
.def pattern 	=r21 ; Bitminta a pergsmentestshez

;*************************************************************** 
;* Reset & Interrupt Vectors  
.cseg 
.org $0000 ; Define start of Code segment 
	jmp RESET ; Reset Handler, jmp is 2 word instruction 
	jmp DUMMY_IT	; Ext. INT0 Handler
	jmp DUMMY_IT	; Ext. INT1 Handler
	jmp DUMMY_IT	; Ext. INT2 Handler
	jmp DUMMY_IT	; Ext. INT3 Handler
	jmp DUMMY_IT	; Ext. INT4 Handler (INT gomb)
	jmp DUMMY_IT	; Ext. INT5 Handler
	jmp DUMMY_IT	; Ext. INT6 Handler
	jmp DUMMY_IT	; Ext. INT7 Handler
	jmp DUMMY_IT	; Timer2 Compare Match Handler 
	jmp DUMMY_IT	; Timer2 Overflow Handler 
	jmp DUMMY_IT	; Timer1 Capture Event Handler 
	jmp DUMMY_IT	; Timer1 Compare Match A Handler 
	jmp DUMMY_IT	; Timer1 Compare Match B Handler 
	jmp DUMMY_IT	; Timer1 Overflow Handler 
	jmp TIMER_IT	; Timer0 Compare Match Handler 
	jmp DUMMY_IT	; Timer0 Overflow Handler 
	jmp DUMMY_IT	; SPI Transfer Complete Handler 
	jmp DUMMY_IT	; USART0 RX Complete Handler 
	jmp DUMMY_IT	; USART0 Data Register Empty Hanlder 
	jmp DUMMY_IT	; USART0 TX Complete Handler 
	jmp DUMMY_IT	; ADC Conversion Complete Handler 
	jmp DUMMY_IT	; EEPROM Ready Hanlder 
	jmp DUMMY_IT	; Analog Comparator Handler 
	jmp DUMMY_IT	; Timer1 Compare Match C Handler 
	jmp DUMMY_IT	; Timer3 Capture Event Handler 
	jmp DUMMY_IT	; Timer3 Compare Match A Handler 
	jmp DUMMY_IT	; Timer3 Compare Match B Handler 
	jmp DUMMY_IT	; Timer3 Compare Match C Handler 
	jmp DUMMY_IT	; Timer3 Overflow Handler 
	jmp DUMMY_IT	; USART1 RX Complete Handler 
	jmp DUMMY_IT	; USART1 Data Register Empty Hanlder 
	jmp DUMMY_IT	; USART1 TX Complete Handler 
	jmp DUMMY_IT	; Two-wire Serial Interface Handler 
	jmp DUMMY_IT	; Store Program Memory Ready Handler 

.org $0046

;****************************************************************
;* DUMMY_IT interrupt handler -- CPU hangup with LED pattern
;* (This way unhandled interrupts will be noticed)

;< tbbi IT kezel a fjl vgre! >

DUMMY_IT:	
	ldi r16,   0xFF ; LED pattern:  *-
	out DDRC,  r16  ;               -*
	ldi r16,   0xA5	;               *-
	out PORTC, r16  ;               -*
DUMMY_LOOP:
	rjmp DUMMY_LOOP ; endless loop

;< tbbi IT kezel a fjl vgre! >

;*************************************************************** 
;* MAIN program, Initialisation part
.org $004B;
RESET: 
;* Stack Pointer init, 
;  Set stack pointer to top of RAM 
	ldi temp, LOW(RAMEND) ; RAMEND = "max address in RAM"
	out SPL, temp 	      ; RAMEND value in "m128def.inc" 
	ldi temp, HIGH(RAMEND) 
	out SPH, temp 

M_INIT:
;< ki- s bemenetek inicializlsa stb > 
	; LED init
	ldi temp, 0xFF		; irny - kimenet
	out DDRC, temp
	ldi temp, 0x00		; egyik sem vilgt
	out PORTC, temp

	; BTN init
	ldi temp, 0x00		; irny - bemenet
	out DDRE, temp

	; TIMER - itt csak pergsmentestshez kell
	ldi temp, 0x0f		; 1024-es elooszt -> 10800Hz
	out TCCR0, temp	
	ldi temp, 107		; 108 ciklus -> 10800/108 = 100Hz
	out OCR0, temp
	ldi temp, 0x02		; TIMER IT engedlyezs
	out TIMSK, temp

	ldi flash_on, 0x01	; villogs engedlyezse
	ldi pressed, 0		; gomb nincs lenyomva
	ldi pattern, 0xff	; bitminta kezdetben csupa 1

	sei			; globlis IT engedlyezs

;*************************************************************** 
;* MAIN program, Endless loop part
 
M_LOOP: 

;< fciklus >
	tst pressed		; test for zero
	breq DO			; ha nincs gombnyoms, ugrunk
	
	clr pressed		; gombnyoms esetn flag trlse	
	com flash_on		; s villogs eng./tilts
	ldi temp, 1		; LED0 vilgtson
	out PORTC, temp		; kijelzs
DO:
	sbrs flash_on, 0	; ha villogs engedlyezve, skip
	jmp M_LOOP		; ha tiltva, ugrs
	
	call DELAY_1S		; 1s vrakozs
	in temp, PORTC		; LED llapot beolvass
	com temp		; vilgt->nem vilgt s ford.
	andi temp, 0x01		; csak az utols bit szmt (LED0)

	out PORTC, temp		; kijelzs

	jmp M_LOOP ; Endless Loop  


;*************************************************************** 
;* Subroutines, Interrupt routines

; 11059200 ~ 250 (cntr2) * 256 (temp) * 57 (cntr1) *3
; mivel egy ciklus vgrehajtsa 3 rajel, ezrt elg a 11059200
;  harmadig elszmolni (ezrt lesz csak 57 a cntr1 kezdortke)
 
DELAY_1S:			; 1 mp SW idozto rutin
	ldi cntr2, 250		; 1s = 250 * 4ms
EXT_LOOP:
	call DELAY_4MS		; 4ms vrakozs
	dec cntr2		; klso szmll cskkentse
	brne EXT_LOOP		; ha nem 0, visszaugrs
	ret			; ha 0, eltelt 1s

DELAY_4MS:			; 4ms idozto rutin
	ldi cntr1, 0x39		; szmll betltse
LOOP_1:
	ldi temp, 0x00		; belso szmll betltse
LOOP_2:
	dec temp		; belso szml. cskkents
	brne LOOP_2		; ha nem 0, tovbb cskkentjk
	dec cntr1		; ha 0, akkor cntr1-et is cskk.
	brne LOOP_1		; ha cntr1 nem 0, belso szml. 
				;  jraindtsa
	ret			; ha mindkt szmll 0, eltelt 4ms

TIMER_IT:			; TIMER IT rutin
	push temp		; ments
	in temp, SREG
	push temp

	; pergsmentests
	in temp, PINE		; gombok beolvassa
	bst temp, 5		; BT0 llapot eltrolsa
	lsl pattern		; bitminta shiftelse balra
	bld pattern, 0		; az LSB helyre beszrjuk 
				; a gomb llapott jelzo bitet

	andi pattern, 0x0f	; felso 4 bitet eldobjuk
	cpi pattern, 0b00001100 ; 1100 bitminta?
	brne NOT_PRESSED	; ha nem, nincs gombnyoms,
				; vagy csak pergst szlel
	inc pressed		; ha igen, tnyleg lefut l

NOT_PRESSED:

	pop temp		; visszallts
	out SREG, temp
	pop temp
	reti
